#
#   python_seguridad_certificado_enviar.py
#
#   Lee datos del cliente, genera certificado y lo envia al servidor
#
#   Requisitos:
#
#   1) Se requiere instalar el conector ODBC de Python a MySQL
#       > pip3 install mysql-connector-python==8.0.29
#
#   2) Se requiere ejecutar el siguiente script en la consola
#       de MySQL para crear un usuario para conectarse
"""
create user 'adriana'@'localhost' identified by '123';
GRANT ALL PRIVILEGES ON * . * TO 'adriana'@'localhost';
"""
# 
#   3) Se requiere ejecutar el siguiente script en la consola
#       de MySQL para construir la base y tabla
"""
create database datos;
use datos;
create table firmas (id text, firma text);
insert into firmas values("01", "ayx123");
create table clima (id text, certificado text, latitud float, longitud float, fecha date, hora time, utc int, variable text, valor float);
"""
#   Rogelio Ferreira Escutia - octubre 2022
#

# Bibliotecas a utilizar
import mysql.connector  # Conector ODBC
import hashlib          # Funciones HASH
import requests         # Enviar peticiones HTTP

def leer_firma(id):
    servidor = "localhost"
    usuario = "adriana"
    clave = "123"
    base = "datos"
    print("Datos de Conexión:")
    print("Servidor=",servidor,"Usuario=",usuario,"Clave=",clave,"Base=",base)
    mi_conexion = mysql.connector.connect(
      host=servidor,
      user=usuario,
      passwd=clave,
      database=base
      )
    try:
        if mi_conexion.is_connected():
            operacion = mi_conexion.cursor()
            operacion.execute( "SELECT * FROM firmas where id=" + id)
            for id, firma in operacion.fetchall() :
                print("id:", id, "firma", firma)
    except Error as e:
        print("Error al conectarse a MySQL: ", e)
    finally:
        mi_conexion.close()
        print("Conexión a la base de datos terminada!")
    return firma

def leer_sensores():
    # Se simula que los siguientes valores se obtienen de sensores
    id = "01"
    latitud = 19.721821     # (Sala "O" del ITM)
    longitud = -101.185778
    fecha = "2022-10-20"
    hora = "14:08"
    utc = -6
    variable = "temperatura"
    valor = 22
    return id, latitud, longitud, fecha, hora, utc, variable, valor

def crear_certificado(firma, hora):
    certificado = (hashlib.md5((firma+hora).encode())).hexdigest()
    return certificado

def almacenar_datos(id, certificado, latitud, longitud, fecha, hora, utc, variable, valor):
    servidor = "localhost"
    usuario = "adriana"
    clave = "123"
    base = "datos"
    print("Datos de Conexión:")
    print("Servidor=",servidor,"Usuario=",usuario,"Clave=",clave,"Base=",base)
    mi_conexion = mysql.connector.connect(
      host=servidor,
      user=usuario,
      passwd=clave,
      database=base
      )
    try:
        if mi_conexion.is_connected():
            operacion = mi_conexion.cursor()
            sql = "INSERT INTO clima (id, certificado, latitud, longitud, fecha, hora, utc, variable, valor) VALUES ('"+id+"','"+certificado+"','"+str(latitud)+"','"+str(longitud)+"','"+fecha+"','"+hora+"',"+str(utc)+",'"+variable+"',"+str(valor)+")"
            print("Sentencia SQL a ejecutar:\n",sql)
            operacion.execute(sql)
            mi_conexion.commit()
    except Error as e:
        print("Error al conectarse a MySQL: ", e)
    finally:
        mi_conexion.close()
        print("Conexión a la base de datos terminada!")
    return firma

def enviar_servidor():
    url_servidor = "http://10.27.34.114/carpeta_servidor/recibir.php"
    parametros = {"id":id, "certificado":certificado, "latitud":latitud, "longitud":longitud, "fecha":fecha, "hora":hora, "utc":utc, "variable":variable, "valor":valor}
    respuesta = requests.post(url_servidor, data = parametros)
    print("Respuesta del servidor:")
    print(respuesta.text)
    return

# Programa Principal
print("\nPROGRAMA PRINCIPAL")

# Leyendo Sensores
print("\n1) Leyendo Sensores...")
id, latitud, longitud, fecha, hora, utc, variable, valor = leer_sensores()

# Leyendo Firma
print("\n2) Leyendo Firma...")
firma = leer_firma(id)
print("El dispositivo",id, "tiene la firma:",firma)

# Creando Certificado
print("\n3) Creando certificado...")
certificado = crear_certificado(firma, hora)
print("El certificado del dispositivo:",id,"con firma:",firma,"es:",certificado)

# Almacenando Datos
print("\n4) Almacenando datos...")
almacenar_datos(id, certificado, latitud, longitud, fecha, hora, utc, variable, valor)

# Enviando los datos al servidor
print("\n5) Enviando los datos al servidor...")
enviar_servidor()
print("\nFIN DEL PROGRAMA\n")